#include <stdlib.h>
#include <math.h>

#include "circle.h"
#include "circle_impl.h"
#include "shape_impl.h"
#include "rectangle.h"
#include "rectangle_impl.h"
#include "symmetric_shape_impl.h"

static int circle_print(const Shape * this, FILE * output);
static void circle_bounding_rectangle(const Shape * this, Shape * rectangle);
static double circle_area(const Shape * this);

/* Constructor
 */
Shape * circle_new(double x, double y, double radius) {
    static struct shape_vtable circle_vtable = {
	.destroy = shape_destroy_impl,
	.print = circle_print,
	.bounding_rectangle = circle_bounding_rectangle,
	.area = circle_area,
	.move = symmetric_shape_move
    };

    Circle * this = malloc(sizeof(Circle));

    if (this) {
	shape_init((Shape *)this, &circle_vtable);
	symmetric_shape_init(&(this->sshape), x, y);
	this->radius = radius;
    }

    return (Shape *)this;
}


/* Methods 
 */
double circle_get_radius(const Shape * s) {
    const Circle * this = (const Circle *)s;
    return this->radius;
}

static int circle_print(const Shape * s, FILE * f) {
    const Circle * this = (const Circle *)s;
    return fprintf(f, "circle: center=(%f,%f) radius=%f",
		   this->sshape.center.x, this->sshape.center.y, this->radius);
}

static void circle_bounding_rectangle(const Shape * s, Shape * r) {
    const Circle * this = (const Circle *)s;
    Rectangle * bbox = (Rectangle *)r;
    bbox->sshape.center.x = this->sshape.center.x;
    bbox->sshape.center.y = this->sshape.center.y;
    bbox->width = 2*this->radius;
    bbox->height = 2*this->radius;
}

static double circle_area(const Shape * s) {
    const Circle * this = (const Circle *)s;
    return 3.14159265 * this->radius * this->radius;
}
