#include <stdio.h>
#include <stdlib.h>

#include "shape.h"
#include "shape_impl.h"
#include "dot.h"
#include "dot_impl.h"
#include "rectangle_impl.h"
#include "coordinates.h"

static void dot_move(Shape * s, double dx, double dy) {
    Dot * this = (Dot *)s;
    this->position.x += dx;
    this->position.y += dy;
}

static const char * dot_type_rep[] = {
    "",				/* DOT_POINT */
    "x",			/* DOT_X */
    "+", 			/* DOT_PLUS */
    "o", 			/* DOT_CIRCLE */
    "<>", 			/* DOT_DIAMON */
};

static int dot_print(const Shape * s, FILE * f) {
    const Dot * this = (const Dot *)s;
    return fprintf(f, "dot: %s (%f,%f)", dot_type_rep[this->type], this->position.x, this->position.y);
}

static void dot_bounding_rectangle(const Shape * s, Shape * r) {
    const Dot * this = (const Dot *)s;
    Rectangle * bbox = (Rectangle *)r;
    bbox->sshape.center.x = this->position.x;
    bbox->sshape.center.y = this->position.y;
    bbox->width = 0;
    bbox->height = 0;
}

static struct shape_vtable dot_vtable = {
    .destroy = shape_destroy_impl,
    .print = dot_print,
    .bounding_rectangle = dot_bounding_rectangle,
    .area = shape_area_impl,
    .move = dot_move };

/* Initializer (private) and constructor (public) */
void dot_init(Dot * this, double x, double y) {
    this->type = DOT_POINT;
    this->position.x = x;
    this->position.y = x;
}

Shape * dot_new(double x, double y) {
    Dot * this = malloc(sizeof(Dot));
    if (this) {
	shape_init(&(this->shape), &dot_vtable);
	dot_init(this, x, y);
    }
    return &(this->shape);
};

enum dot_type dot_get_type(const Shape * s) {
    const Dot * this = (const Dot *)s;
    return this->type;
}

void dot_set_type(Shape * s, enum dot_type t) {
    Dot * this = (Dot *)s;
    this->type = t;
}

Coordinates * dot_get_position(Shape * s) {
    Dot * this = (Dot *)s;
    return &(this->position);
}

