#include <stdlib.h>

#include "shape_impl.h"
#include "rectangle.h"
#include "rectangle_impl.h"
#include "symmetric_shape_impl.h"

static int rectangle_print(const Shape * this, FILE * output);
static void rectangle_bounding_rectangle(const Shape * this, Shape * rectangle);
static double rectangle_area(const Shape * this);
static void rectangle_move(Shape * this, double dx, double dy);

static double rectangle_get_width_impl(const Shape * this);
static double rectangle_get_height_impl(const Shape * this);

/* Constructor
 */
Shape * rectangle_new(double x, double y, double w, double h) {
    static struct rectangle_vtable rectangle_vtable = {
	.shape_vt = {
	    .destroy = shape_destroy_impl,
	    .print = rectangle_print,
	    .bounding_rectangle = rectangle_bounding_rectangle,
	    .area = rectangle_area,
	    .move = symmetric_shape_move },
	.get_width = rectangle_get_width_impl,
	.get_height = rectangle_get_height_impl,
    };

    Rectangle * this = malloc(sizeof(Rectangle));

    if (this) {
	shape_init((Shape *)this, &(rectangle_vtable.shape_vt));
	symmetric_shape_init(&(this->sshape), x, y);
	this->width = w;
	this->height = h;
    }

    return (Shape *)this;
}


/* Methods 
 */
double rectangle_get_width(const Shape * this) {
    return ((struct rectangle_vtable *)(this->vt))->get_width(this);
}

double rectangle_get_height(const Shape * this) {
    return ((struct rectangle_vtable *)(this->vt))->get_height(this);
}

static double rectangle_get_width_impl(const Shape * s) {
    const Rectangle * this = (const Rectangle *)s;
    return this->width;
}

static double rectangle_get_height_impl(const Shape * s) {
    const Rectangle * this = (const Rectangle *)s;
    return this->height;
}

static int rectangle_print(const Shape * s, FILE * f) {
    const Rectangle * this = (const Rectangle *)s;
    return fprintf(f, "rectangle: center=(%f,%f) width=%f height=%f",
		   this->sshape.center.x, this->sshape.center.y,
		   this->width, this->height);
}

static void rectangle_bounding_rectangle(const Shape * s, Shape * r) {
    const Rectangle * this = (const Rectangle *)s;
    Rectangle * bbox = (Rectangle *)r;
    bbox->sshape.center.x = this->sshape.center.x;
    bbox->sshape.center.y = this->sshape.center.y;
    bbox->width = this->width;
    bbox->height = this->height;
}

static double rectangle_area(const Shape * s) {
    const Rectangle * this = (const Rectangle *)s;
    return this->width * this->height;
}
