#include <stdio.h>
#include <stdlib.h>

#include "shape.h"
#include "shape_impl.h"
#include "rectangle_impl.h"

/* Virtual methods 
*/
void shape_destroy(Shape * this) {
    this->vt->destroy(this);
}

int shape_print(const Shape * this, FILE * output) {
    return this->vt->print(this, output);
}

void shape_bounding_rectangle(const Shape * this, Shape * rectangle) {
    this->vt->bounding_rectangle(this, rectangle);
}

double shape_area(const Shape * this) {
    return this->vt->area(this);
}

void shape_move(Shape * this, double dx, double dy) {
    this->vt->move(this, dx, dy);
}

/* Non virtual methods 
*/
int shape_get_id(const Shape * this) {
    return this->id;
}

/* Private methods 
*/
void shape_init(Shape * this, struct shape_vtable * t) {
    static int id_counter = 0;
    this->id = ++id_counter;
    this->vt = t;
}

double shape_area_impl(const Shape * this) {
    return 0;
}

void shape_destroy_impl(Shape * this) {
    free(this);
}

