#ifndef SHAPE_H_INCLUDED
#define SHAPE_H_INCLUDED

#include <stdio.h>

/* This defines an abstract class called Shape that is also the root
 * of a hierarchy of classes representing graphical objects.
 *
 * Here we only declare a struct shape, which is opaque to the user.
 * This means that the user may only use pointers to Shape objects.
 */
struct shape;
typedef struct shape Shape;

/* Constructors: We have no constructors in this case, since this is
 * an abstract class and therefore there will be no objects of type
 * Shape.
 */

/* Destructor: the destructor is instead here, and it is also virtual,
 * as it should be.
 */
extern void shape_destroy(Shape * this);

/* Methods: these are object-level methods (as opposed to "static" or
 * class-level methods), which means that they all take a Shape object
 * reference (a pointer) as a parameter.
 *
 * Notice that these are virtual or non-virtual methods.  There is
 * actually no difference between virtual and non-virtual methods at
 * the API level (here).  See the implementation to figure out which
 * methods are virtual and which ones are not.
 */
extern int shape_get_id(const Shape * this); 
extern int shape_print(const Shape * this, FILE * output);
extern void shape_bounding_rectangle(const Shape * this, Shape * rectangle);
extern double shape_area(const Shape * this);
extern void shape_move(Shape * this, double dx, double dy);

#endif
