#include <stdlib.h>

#include "square.h"
#include "square_impl.h"
#include "shape_impl.h"
#include "rectangle.h"
#include "rectangle_impl.h"
#include "symmetric_shape_impl.h"

static int square_print(const Shape * this, FILE * output);
static void square_bounding_rectangle(const Shape * this, Shape * rectangle);
static double square_area(const Shape * this);

/* Constructor
 */
Shape * square_new(double x, double y, double side) {
    static struct rectangle_vtable square_vtable = {
	.shape_vt = {
	    .destroy = shape_destroy_impl,
	    .print = square_print,
	    .bounding_rectangle = square_bounding_rectangle,
	    .area = square_area,
	    .move = symmetric_shape_move },
	.get_width = square_get_side,
	.get_height = square_get_side,
    };

    Square * this = malloc(sizeof(Square));

    if (this) {
	shape_init((Shape *)this, (struct shape_vtable *)(&square_vtable));
	symmetric_shape_init(&(this->sshape), x, y);
	this->side = side;
    }

    return (Shape *)this;
}


/* Methods 
 */
double square_get_side(const Shape * s) {
    const Square * this = (const Square *)s;
    return this->side;
}

static int square_print(const Shape * s, FILE * f) {
    const Square * this = (const Square *)s;
    return fprintf(f, "square: center=(%f,%f) side=%f",
		   this->sshape.center.x, this->sshape.center.y, this->side);
}

static void square_bounding_rectangle(const Shape * s, Shape * r) {
    const Square * this = (const Square *)s;
    Rectangle * bbox = (Rectangle *)r;
    bbox->sshape.center.x = this->sshape.center.x;
    bbox->sshape.center.y = this->sshape.center.y;
    bbox->width = this->side;
    bbox->height = this->side;
}

static double square_area(const Shape * s) {
    const Square * this = (const Square *)s;
    return this->side * this->side;
}
